/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PackageHandler.java 154 2 mai 2006 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.zip.ZipFile;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.ZipUtil;
import org.objectweb.util.monolog.api.Logger;
import org.xml.sax.SAXException;

/**
 * Utility class for handling packages and package URIs
 * 
 * @author Olivier Fabre - EBM Websourcing
 * 
 */
public class PackageHandler {

    private LoggingUtil log;

    private Logger logger;

    private RepositoryService repositorySrv;

    public PackageHandler(Logger logger, RepositoryService repositorySrv) {
        super();
        this.logger = logger;
        this.log = new LoggingUtil(logger);
        this.repositorySrv = repositorySrv;
    }

    /**
     * Expand the given package into the system repository
     * 
     * @param uuid
     * @param zipArchiveURI
     * @return return installation URI of the expanded package, null if it fails
     */
    public URI expandPackageIntoRepository(String uuid, URI zipArchiveURI) {
        log.start();

        String msg;
        URI installationRoot = null;

        /*
         * explode the given package Zip archive into platform repository
         */

        try {
            installationRoot = repositorySrv.addPackage(uuid, zipArchiveURI);
        } catch (PetalsException pe) {
            msg = "Unexpected error adding JBI installation "
                + "package into repository. " + pe.getMessage();
            log.error(msg, pe);
        }

        if (installationRoot == null) {
            msg = "Unexpected Platform repository service error. The "
                + "installation root must NOT be null: " + uuid;
            log.error(msg);

            /*
             * don't keep incomplete package installation into repository
             */
            repositorySrv.removePackage(uuid);
        }
        log.end();
        return installationRoot;
    }

    /**
     * Load the jbi descriptor from the given archive
     * 
     * @param archiveZipURL
     * @return
     * @throws PetalsException
     */
    public JBIDescriptor loadDescriptor(URI archiveURI) throws PetalsException {
        log.start();

        String msg;
        JBIDescriptor result = null;

        /*
         * extract descriptor file to parse and validate
         */

        File tmpJbiXml = null;

        try {
            /*
             * open the package Zip archive file and extract jbi.xml temp
             */
            try {
                ZipFile zipArchive = ZipUtil.openZipFile(archiveURI);

                tmpJbiXml = ZipUtil.getEntryAsTemp(zipArchive,
                    JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE);

                zipArchive.close();
            } catch (IOException e) {
                msg = "Error during jbi.xml extraction from package : "
                    + archiveURI.getPath();
                log.error(msg);

                throw new PetalsException(msg, e);
            }

            if (tmpJbiXml == null) {
                msg = "Invalid JBI package Zip archive. jbi.xml descriptor "
                    + "entry was not found: " + archiveURI.toString();
                log.error(msg);

                throw new PetalsException(msg);
            }

            /*
             * use builder to parse and validate jbi.xml against the schema
             */

            JBIDescriptorBuilder descBuilder = new JBIDescriptorBuilder(
                tmpJbiXml.toURI(), (java.util.logging.Logger) logger);

            result = descBuilder.build();

        } catch (SAXException saxe) {
            msg = "XML error parsing JBI descriptor. Package archive: "
                + archiveURI.toString();

            log.error(msg, saxe);

            throw new PetalsException(msg, saxe);
        } finally {
            /*
             * delete temporary file
             */
            if (tmpJbiXml != null) {
                tmpJbiXml.delete();
            }
        }
        log.end();

        return result;
    }

    /**
     * Process the given package Zip archive URL string and transform into a
     * ready-to-use URI object. Additionally, checks if the point specified
     * exist.
     * 
     * @return <code>URI</code> the proper package URI.
     * @throws PetalsException
     *             wrong package ZIP archive URL were specified.
     */
    public URI processAndGetPackageURI(String archiveZipURL,
        boolean checkIfExist) throws PetalsException {
        log.start(archiveZipURL);

        String msg;
        URI uri = null;

        /*
         * Create URI
         */

        try {
            URL url = new URL(archiveZipURL);

            uri = url.toURI();
        } catch (MalformedURLException mue) {
            msg = "Wrong package archive location specified. URL: "
                + archiveZipURL;

            log.error(msg, mue);

            throw new PetalsException(msg, mue);
        } catch (URISyntaxException use) {
            msg = "Syntax error converting to URI from given location. URL: "
                + archiveZipURL;
            log.error(msg, use);

            throw new PetalsException(msg, use);
        }

        /*
         * check if exists
         */
        if (checkIfExist) {
            File archive = new File(uri);

            if (!archive.exists()) {
                msg = "JBI package archive not found. URL: " + archiveZipURL;
                log.error(msg);

                throw new PetalsException(msg);
            }
        }
        log.end();

        return uri;
    }

}
