/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemRecovery.java 154 2 mai 2006 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.systemstate;

import java.net.URI;
import java.util.List;

import javax.jbi.JBIException;
import javax.jbi.management.LifeCycleMBean;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.lifecycle.Installer;
import org.objectweb.petals.jbi.management.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.service.DeploymentServiceMBean;
import org.objectweb.petals.jbi.management.service.InstallationServiceMBean;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerService;
import org.objectweb.petals.jbi.management.service.ManagementException;
import org.objectweb.petals.jbi.management.service.PackageHandler;
import org.objectweb.petals.jbi.management.service.util.XMLResult;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Allows to restore system state after shutdown or crash.
 * 
 * @author Olivier Fabre - EBM Websourcing
 * 
 */
public class SystemRecovery {

    protected PackageHandler packageHandler;

    protected DeploymentServiceMBean deploymentService;

    protected InstallationServiceMBean installationService;

    protected LoggingUtil log;

    protected LifeCycleManagerService managerService;

    protected StateStorage stateStorage;

    public SystemRecovery(InstallationServiceMBean installationService,
        Logger logger, DeploymentServiceMBean deploymentService,
        LifeCycleManagerService managerService, StateStorage stateStorage,
        RepositoryService repositorySrv) {
        super();
        this.deploymentService = deploymentService;
        this.installationService = installationService;
        this.managerService = managerService;
        this.log = new LoggingUtil(logger);
        this.stateStorage = stateStorage;
        this.packageHandler = new PackageHandler(logger, repositorySrv);
    }

    public boolean recoverAllComponent() throws ManagementException {
        log.start();
        String msg = null;

        /*
         * Retrieve all shared library to recover
         */

        List<ComponentState> cps = stateStorage.retrieveAllComponentStates();

        /*
         * Recover state for each component
         */
        for (ComponentState state : cps) {
            try {

                URI installationRoot = packageHandler.processAndGetPackageURI(
                    state.getInstallURL(), true);

                URI archiveURI = packageHandler.processAndGetPackageURI(state
                    .getArchiveURL(), true);

                JBIDescriptor descriptor = packageHandler
                    .loadDescriptor(archiveURI);

                recoverComponent(installationRoot, descriptor, state
                    .getInstallState(), state.getLifecycleState());
            } catch (PetalsException e) {
                msg = "A component can't be recovered : " + state.getName();
                log.error(msg);
                throw new ManagementException(msg, e);
            }
        }
        log.end();
        return false;
    }

    public boolean recoverAllServiceAssembly() throws ManagementException {
        log.start();
        String msg = null;

        /*
         * Retrieve all service assembly to recover
         */

        List<ServiceAssemblyState> sas = stateStorage
            .retrieveAllServiceAssemblyStates();

        /*
         * Recover state for each service assembly
         */
        for (ServiceAssemblyState state : sas) {
            try {

                URI installationRoot = packageHandler.processAndGetPackageURI(
                    state.getInstallURL(), true);

                URI archiveURI = packageHandler.processAndGetPackageURI(state
                    .getArchiveURL(), true);

                JBIDescriptor descriptor = packageHandler
                    .loadDescriptor(archiveURI);

                recoverServiceAssembly(installationRoot, descriptor, state
                    .getLifecycleState());
            } catch (PetalsException e) {
                msg = "A service assembly can't be recovered : "
                    + state.getName();
                log.error(msg);
                throw new ManagementException(msg, e);
            }
        }
        log.end();
        return false;
    }

    public boolean recoverAllSharedLibrary() throws ManagementException {
        log.start();
        String msg = null;

        /*
         * Retrieve all shared library to recover
         */

        List<SharedLibraryState> sls = stateStorage
            .retrieveAllSharedLibraryStates();

        /*
         * Recover state for each shared library
         */
        for (SharedLibraryState state : sls) {
            try {

                URI installationRoot = packageHandler.processAndGetPackageURI(
                    state.getInstallURL(), true);

                URI archiveURI = packageHandler.processAndGetPackageURI(state
                    .getArchiveURL(), true);

                JBIDescriptor descriptor = packageHandler
                    .loadDescriptor(archiveURI);

                recoverSharedLib(installationRoot, descriptor);
            } catch (PetalsException e) {
                msg = "A Shared lib can't be recovered : " + state.getName();
                log.error(msg);
                throw new ManagementException(msg, e);
            }
        }
        log.end();
        return false;
    }

    /**
     * Recover the given component. Recovering a component consist in redoing
     * installation process of a component (see <code>InstallationService</code>),
     * calling the "install" method on the created Installer and putting the
     * component in its previous state by calling "start" or/and "stop" on the
     * created component lifecycle.
     * 
     * @param installationRoot
     * @param archiveURI
     * @param installState
     * @param lifecycleState
     * @throws ManagementException
     */
    protected void recoverComponent(URI installationRoot,
        JBIDescriptor descriptor, String installState, String lifecycleState)
        throws ManagementException {
        log.start();
        String msg = null;

        String compName = descriptor.getComponent().getIdentification()
            .getName();
        try {

            if (Installer.INSTALLED.equals(installState)) {
                /*
                 * Redo install
                 */
                ObjectName installerName = installationService
                    .loadNewInstaller(installationRoot, descriptor);
                MBeanServer beanServer = managerService.getMBeanServer();

                Object[] nullObjects = new Object[0];
                String[] nullStrings = new String[0];
                ObjectName clcName = (ObjectName) beanServer.invoke(
                    installerName, "install", nullObjects, nullStrings);

                if (LifeCycleMBean.STOPPED.equals(lifecycleState)) {
                    /*
                     * Redo start followed by stop
                     */
                    beanServer.invoke(clcName, "start", nullObjects,
                        nullStrings);
                    beanServer
                        .invoke(clcName, "stop", nullObjects, nullStrings);
                } else if (LifeCycleMBean.STARTED.equals(lifecycleState)) {
                    /*
                     * Redo start
                     */
                    beanServer.invoke(clcName, "start", nullObjects,
                        nullStrings);
                }
            }
        } catch (InstanceNotFoundException e) {
            msg = "Component can't be recovered : " + compName;
            log.error(msg);
            throw new ManagementException(msg, e);
        } catch (MBeanException e) {
            msg = "Component can't be recovered : " + compName;
            log.error(msg);
            throw new ManagementException(msg, e);
        } catch (ReflectionException e) {
            msg = "Component can't be recovered : " + compName;
            log.error(msg);
            throw new ManagementException(msg, e);
        }
        log.end();
    }

    /**
     * Recover the given Service Assembly. Recovering a Service Assembly consist
     * in redoing deployment process of a Service Assembly (see
     * <code>DeploymentService</code>) and putting the Service Assembly in
     * its previous state by calling "start" or/and "stop" on the created
     * component lifecycle.
     * 
     * @param installationRoot
     * @param archiveURI
     * @param lifecycleState
     * @throws ManagementException
     */
    protected void recoverServiceAssembly(URI installationRoot,
        JBIDescriptor descriptor, String lifecycleState)
        throws ManagementException {
        log.start();
        String msg = null;

        String saName = descriptor.getServiceAssembly().getIdentification()
            .getName();
        try {

            /*
             * Redo deployment
             */

            String xml = deploymentService.deploy(installationRoot, descriptor);
            if (xml.indexOf(XMLResult.TaskResult.FAILED.toString()) > -1) {
                log.warning("Problem while deploying a service assembly\n"
                    + xml);
            }

            if (LifeCycleMBean.STOPPED.equals(lifecycleState)) {
                /*
                 * Redo start followed by stop
                 */
                ServiceAssemblyLifeCycle salc = managerService
                    .getServiceAssemblyByName(saName);
                salc.start();
                salc.stop();
            } else if (LifeCycleMBean.STARTED.equals(lifecycleState)) {
                /*
                 * Redo start
                 */
                ServiceAssemblyLifeCycle salc = managerService
                    .getServiceAssemblyByName(saName);
                salc.start();
            }

        } catch (PetalsException e) {
            msg = "ServiceAssembly can't be recovered : " + saName;
            log.error(msg);
            throw new ManagementException(msg, e);
        } catch (JBIException e) {
            msg = "ServiceAssembly can't be recovered : " + saName;
            log.error(msg);
            throw new ManagementException(msg, e);
        }
        log.end();
    }

    /**
     * Recover the given shared library. Recovering a shared lib consist in
     * redoing installation process of a shared lib (see
     * <code>InstallationService</code>).
     * 
     * @param installationRoot
     * @param archiveURI
     * @throws ManagementException
     */
    protected void recoverSharedLib(URI installationRoot,
        JBIDescriptor descriptor) throws ManagementException {
        log.start();

        /*
         * Redo install
         */
        installationService.installSharedLibrary(installationRoot, descriptor);

        log.end();
    }

}
