/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemStateImpl.java 154 26 avr. 2006 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.systemstate;

import java.util.List;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.jbi.management.service.DeploymentServiceMBean;
import org.objectweb.petals.jbi.management.service.InstallationServiceMBean;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerService;
import org.objectweb.petals.jbi.management.service.ManagementException;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_DEPLOYMENT_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_INSTALLATION_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.MONOLOG_FACTORY_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;

/**
 * Allows to persist component, shared library and service assembly states and
 * restore them after petals system shutdown or crash.
 * 
 * @author Olivier Fabre - EBM Websourcing
 */
public class SystemStateImpl implements SystemState, LifeCycleController,
    BindingController {

    private static final String[] CLIENT_INTERFACES = new String[] {
        JBI_DEPLOYMENT_NAME, JBI_INSTALLATION_NAME, JBI_LIFECYCLEMANAGER_NAME,
        PLATFORM_REPOSITORY_NAME};

    /**
     * JBI Container :: Deployment Service
     */
    protected DeploymentServiceMBean deploymentService;

    /**
     * JBI Container :: Installation Service
     */
    protected InstallationServiceMBean installationService;

    /**
     * Component lifecycle state.
     */
    protected String fcState;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog Logger instance.
     */
    protected Logger logger;

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected LifeCycleManagerService managerService;

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositorySrv;

    protected StateStorage stateStorage;

    protected SystemRecovery systemRecovery;

    /**
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
     *      java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf)
        throws NoSuchInterfaceException, IllegalBindingException,
        IllegalLifeCycleException {

        if (LOGGER_NAME.equals(clientItfName)) {
            logger = (Logger) serverItf;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else if (JBI_DEPLOYMENT_NAME.equals(clientItfName)) {
            deploymentService = (DeploymentServiceMBean) serverItf;
        } else if (JBI_INSTALLATION_NAME.equals(clientItfName)) {
            installationService = (InstallationServiceMBean) serverItf;
        } else if (JBI_LIFECYCLEMANAGER_NAME.equals(clientItfName)) {
            managerService = (LifeCycleManagerService) serverItf;
        } else if (PLATFORM_REPOSITORY_NAME.equals(clientItfName)) {
            repositorySrv = (RepositoryService) serverItf;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    public ComponentState createComponentStateHolder(String componentName,
        String installURL, String zipURL, String installState,
        String lifecycleState) throws Exception {
        return stateStorage.createComponentStateHolder(componentName,
            installURL, zipURL, installState, lifecycleState);
    }

    public ServiceAssemblyState createServiceAssemblyStateHolder(String saName,
        String installURL, String zipURL, String lifecycleState)
        throws Exception {
        return stateStorage.createServiceAssemblyStateHolder(saName,
            installURL, zipURL, lifecycleState);
    }

    public SharedLibraryState createSharedLibraryStateHolder(String slName,
        String installURL, String zipURL) throws Exception {
        return stateStorage.createSharedLibraryStateHolder(slName, installURL,
            zipURL);
    }

    public ComponentState deleteComponentStateHolder(String componentName)
        throws Exception {
        return stateStorage.deleteComponentStateHolder(componentName);
    }

    public ServiceAssemblyState deleteServiceAssemblyStateHolder(String saName)
        throws Exception {
        return stateStorage.deleteServiceAssemblyStateHolder(saName);
    }

    public SharedLibraryState deleteSharedLibraryStateHolder(String slName)
        throws Exception {
        return stateStorage.deleteSharedLibraryStateHolder(slName);
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
     */
    public String getFcState() {
        return STARTED.equals(fcState) ? STARTED : STOPPED;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return CLIENT_INTERFACES;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName)
        throws NoSuchInterfaceException {

        if (JBI_DEPLOYMENT_NAME.equals(clientItfName)) {
            return deploymentService;
        } else if (JBI_INSTALLATION_NAME.equals(clientItfName)) {
            return installationService;
        } else if (JBI_LIFECYCLEMANAGER_NAME.equals(clientItfName)) {
            return managerService;
        } else if (PLATFORM_REPOSITORY_NAME.equals(clientItfName)) {
            return repositorySrv;
        } else if (LOGGER_NAME.equals(clientItfName)) {
            return logger;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    public boolean recoverAllComponent() throws ManagementException {
        return systemRecovery.recoverAllComponent();
    }

    public boolean recoverAllEntities() {
        log.start();
        boolean result = false;

        try {
            recoverAllSharedLibrary();
            recoverAllComponent();
            recoverAllServiceAssembly();
            result = true;
        } catch (ManagementException e) {
            String msg = "Entities can't be correctly recovered !";
            log.error(msg, e);
        }
        log.end();
        return result;
    }

    public boolean recoverAllServiceAssembly() throws ManagementException {
        return systemRecovery.recoverAllServiceAssembly();
    }

    public boolean recoverAllSharedLibrary() throws ManagementException {
        return systemRecovery.recoverAllSharedLibrary();
    }

    public List<ComponentState> retrieveAllComponentStates() {
        return stateStorage.retrieveAllComponentStates();
    }

    public List<ServiceAssemblyState> retrieveAllServiceAssemblyStates() {
        return stateStorage.retrieveAllServiceAssemblyStates();
    }

    // ----------------------------------------------------------------------
    // Fractal BindingController implementation
    // ----------------------------------------------------------------------

    public List<SharedLibraryState> retrieveAllSharedLibraryStates() {
        return stateStorage.retrieveAllSharedLibraryStates();
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    public void startFc() throws IllegalLifeCycleException {

        init();

        fcState = STARTED;
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
     */
    public void stopFc() throws IllegalLifeCycleException {

        shutdown();

        fcState = STOPPED;
    }

    /**
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {

        if (LOGGER_NAME.equals(clientItfName)) {
            logger = null;
        } else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
            // nothing
        } else if (JBI_DEPLOYMENT_NAME.equals(clientItfName)) {
            deploymentService = null;
        } else if (JBI_INSTALLATION_NAME.equals(clientItfName)) {
            installationService = null;
        } else if (JBI_LIFECYCLEMANAGER_NAME.equals(clientItfName)) {
            managerService = null;
        } else if (PLATFORM_REPOSITORY_NAME.equals(clientItfName)) {
            repositorySrv = null;
        } else {
            throw new NoSuchInterfaceException(clientItfName);
        }
    }

    // ----------------------------------------------------------------------
    // Fractal LifecycleController implementation
    // ----------------------------------------------------------------------

    public void updateComponentInstallationState(String componentName,
        String installState) throws Exception {
        stateStorage.updateComponentInstallationState(componentName,
            installState);

    }

    public void updateComponentLifeCycleState(String componentName,
        String lifecycleState) throws Exception {
        stateStorage.updateComponentLifeCycleState(componentName,
            lifecycleState);

    }

    public void updateServiceAssemblyState(String saName, String lifecycleState)
        throws Exception {
        stateStorage.updateServiceAssemblyState(saName, lifecycleState);

    }

    protected void init() {
        log = new LoggingUtil(logger);
        log.start();

        stateStorage = new StateStorage();

        try {
            stateStorage.init();
        } catch (Exception e) {
            String msg = "SystemState component can't be correctly initiated : "
                + "StateStorage init failed !";
            log.error(msg, e);
            throw new RuntimeException(msg);
        }

        systemRecovery = new SystemRecovery(installationService, logger,
            deploymentService, managerService, stateStorage, repositorySrv);

        log.end();
    }

    protected void shutdown() {
        log.start();

        logger = null;

        log.end();
    }
}
