/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DistributedJNDIServer.java 6:42:03 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.registry;

import java.net.InetAddress;
import java.util.Collections;
import java.util.Map;
import java.util.TreeMap;

import javax.naming.NamingException;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.kernel.configuration.ConfigurationService;
import org.objectweb.petals.kernel.registry.msg.request.RegistryRequest;
import org.objectweb.petals.kernel.registry.msg.response.RegistryResponse;
import org.objectweb.petals.kernel.registry.thread.ElectMasterThread;
import org.objectweb.petals.kernel.registry.thread.SocketServerThread;
import org.objectweb.petals.kernel.registry.thread.TribeThread;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.tribe.channel.AbstractReliableFifoChannel;
import org.objectweb.tribe.channel.ReliableGroupChannelWithGms;
import org.objectweb.tribe.channel.tcp.TcpChannelPool;
import org.objectweb.tribe.common.GroupIdentifier;
import org.objectweb.tribe.common.IpAddress;
import org.objectweb.tribe.gms.GroupMembershipService;
import org.objectweb.tribe.gms.discovery.UdpDiscoveryService;
import org.objectweb.util.monolog.api.Logger;

/**
 * JNDI Server
 * 
 * At startup :
 * <ol>
 * <li>The JNDI server will start the socket server and the tribe listener.</li>
 * <li>The JNDI server will search for other members in the group by sending a
 * electMaster request</li>
 * <li>
 * <ul>
 * <li>If the server is alone then it will be its own master (by keeping its
 * master field at null)</li>
 * <li>If there is a master, it will send back its member id and ask for a full
 * update of its data</li>
 * </ul>
 * </li>
 * </ol>
 * 
 * At Shutdown :
 * <ul>
 * <li>If the server is not the master, the server does nothing</li>
 * <li>If the server is the master :
 * <ol>
 * <li>The server a newMaster request to all the servers</li>
 * <li>The others servers reset their master field</li>
 * <li>The others servers sends a electNewMaster message with their starttime</li>
 * <li>Each server checks if it is the master (the one with the smallest start
 * time)</li>
 * </ol>
 * </li>
 * </ul>
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class RegistryServer implements LifeCycleController, BindingController {

    /**
     * Group channel
     * 
     * The send method is broadcast thus we have to take care when we send a
     * message because we receive our own messages if we use the send method
     * without parameters
     */
    protected ReliableGroupChannelWithGms channel;

    /**
     * Data stored in this registry
     */
    protected Map<String, Map<String, Object>> data = Collections
        .synchronizedMap(new TreeMap<String, Map<String, Object>>());

    /**
     * Discovery service
     */
    protected UdpDiscoveryService discovery;

    /**
     * Configuration initializer thread
     */
    protected ElectMasterThread electMasterThread;

    /**
     * Factral state of the component
     */
    protected String fcState = LifeCycleController.STOPPED;

    /**
     * Group membership service
     */
    protected GroupMembershipService gms;

    /**
     * Name of the Tribe group
     */
    protected String groupName = SystemUtil.getJoramDomain();

    protected boolean isMasterSet;

    /**
     * Boolean to know if a server is started
     */
    protected boolean isStarted;

    /**
     * Boolean to know if the server has been updated
     */
    protected boolean fullyUpdated;

    /**
     * Logger
     */
    protected LoggingUtil log;

    /**
     * Logger
     */
    protected Logger logger;

    /**
     * Hashcode of the master
     */
    protected long master;

    /**
     * Registry Util
     */
    protected RegistryUtil registryUtil;

    /**
     * Socket server thread
     */
    protected SocketServerThread socketServerThread;

    /**
     * Start time (to find who is the master)
     */
    protected long startTime = System.currentTimeMillis();

    /**
     * Tribe thread
     */
    protected TribeThread tribeThread;

    public RegistryServer() {
        super();
        try {
            InetAddress multicastAddress = InetAddress.getByName(SystemUtil
                .getTribeMulticastAddress());
            final int multicastPort = 2288;
            final IpAddress multicastIP = new IpAddress(multicastAddress,
                multicastPort);
            final InetAddress repplyAddress = InetAddress.getByName(SystemUtil
                .getHost());
            int repplyPort = 0;
            final IpAddress repplyIP = new IpAddress(repplyAddress, repplyPort);

            discovery = new UdpDiscoveryService(multicastIP, repplyIP);
            gms = new GroupMembershipService(repplyIP, TcpChannelPool
                .getChannelPool(), discovery);

            channel = new ReliableGroupChannelWithGms(gms);

            data.put("/", new TreeMap<String, Object>());

        } catch (Exception e) {
            // Do nothing
        }
        registryUtil = new RegistryUtil(this);
    }

    /**
     * @see BindingController#bindFc(String, Object)
     */
    public void bindFc(String arg0, Object arg1)
        throws NoSuchInterfaceException, IllegalBindingException,
        IllegalLifeCycleException {
        if (arg0.equals(InterfaceNamesHelper.LOGGER_NAME)) {
            logger = (Logger) arg1;
            log = new LoggingUtil(logger);
            registryUtil.setLog(log);
        } else {
            throw new NoSuchInterfaceException(arg0);
        }
    }

    public ReliableGroupChannelWithGms getChannel() {
        return channel;
    }

    public Map<String, Map<String, Object>> getData() {
        return data;
    }

    /**
     * Fractal client interfaces
     */
    private static final String[] CLIENT_INTERFACES = new String[] {};

    /**
     * @see LifeCycleController#getFcState()
     */
    public String getFcState() {
        return fcState.equals(LifeCycleController.STARTED) ? LifeCycleController.STARTED
            : LifeCycleController.STOPPED;
    }

    public LoggingUtil getLog() {
        return log;
    }

    public long getMaster() {
        return master;
    }

    public long getStartTime() {
        return startTime;
    }

    public boolean isMasterSet() {
        return isMasterSet;
    }

    /**
     * @see BindingController#listFc()
     */
    public String[] listFc() {
        return CLIENT_INTERFACES;
    }

    /**
     * Configuration service
     */
    protected ConfigurationService configurationService;

    /**
     * @see BindingController#lookupFc(String)
     */
    public Object lookupFc(String arg0) throws NoSuchInterfaceException {
        Object out = null;
        if (arg0.equals(InterfaceNamesHelper.LOGGER_NAME)) {
            out = logger;
        } else {
            throw new NoSuchInterfaceException(arg0);
        }
        return out;
    }

    /**
     * Process a request
     * 
     * @param request
     *            request to process
     * @return output Object of the operation
     * @throws NamingException
     */
    public Object processRequest(RegistryRequest request)
        throws NamingException {
        // log.call();
        // log.debug(startTime + " : " + request.getType());
        Object output = null;
        switch (request.getType()) {
            /**
             * BIND
             */
            case bind:
                registryUtil.bind(request.getArg1(), request.getArg2(), request
                    .getArg3());
                break;
            /**
             * CREATE SUBCONTEXT
             */
            case createSubcontext:
                output = registryUtil.createSubcontext(request.getArg1(),
                    request.getArg2());
                break;
            /**
             * DESTROY SUBCONTEXT
             */
            case destroySubcontext:
                registryUtil.destroySubcontext(request.getArg1(), request
                    .getArg2());
                break;
            /**
             * ELECT MASTER
             */
            case electMaster:
                if (master == 0
                    && startTime < (Long) request.getArg1()) {
                    // The local server has started before the
                    // requester
                    log.debug(startTime
                        + " This server is the master");
                    output = startTime;
                }
                break;
            /**
             * ELECT NEW MASTER
             */
            case electNewMaster:
                // We reset the master
                master = 0;
                break;
            /**
             * FULL UPDATE
             */
            case fullUpdate:
                output = data;
                break;
            /**
             * LIST
             */
            case list:
                output = registryUtil
                    .list(request.getArg1(), request.getArg2());
                break;
            /**
             * LIST BINDINGS
             */
            case listBindings:
                output = registryUtil.listBindings(request.getArg1(), request
                    .getArg2());
                break;
            /**
             * LOOKUP
             */
            case lookup:
                output = registryUtil.lookup(request.getArg1(), request
                    .getArg2());
                break;
            /**
             * LOOKUP LINK
             */
            case lookupLink:
                output = registryUtil.lookupLink(request.getArg1(), request
                    .getArg2());
                break;
            /**
             * REBIND
             */
            case rebind:
                registryUtil.rebind(request.getArg1(), request.getArg2(),
                    request.getArg3());
                break;
            /**
             * RENAME
             */
            case rename:
                registryUtil.rename(request.getArg1(), request.getArg2(),
                    request.getArg3());
                break;
            /**
             * UNBIND
             */
            case unbind:
                registryUtil.unbind(request.getArg1(), request.getArg2());
                break;
            default:
                break;
        }
        // log.end();
        return output;
    }

    /**
     * Process a response
     * 
     * @param response
     *            Reponse to process
     * @return output of the corresponding operation
     */
    @SuppressWarnings("unchecked")
    public Object processResponse(RegistryResponse response) {
        // log.call();
        // log.debug(startTime + " " + response.getType());
        Object output = null;
        switch (response.getType()) {
            /**
             * FULL UPDATE
             */
            case fullUpdate:
                fullyUpdated = true;
                data = (Map<String, Map<String, Object>>) response.getArg1();
                break;
            /**
             * MASTER
             */
            case master:
                isMasterSet = true;
                master = (Long) response.getArg1();
                break;
            /**
             * MY STARTIME
             */
            case myStartTime:
                if ((Long) response.getArg1() > startTime) {
                    output = "i can be a master";
                }
                break;
            /**
             * EXCEPTION
             */
            case exception:
                // log.debug(this.startTime + " EXCEPTION : " +
                // response.getArg1());
                break;
            default:
                log.warning(this.startTime
                    + " Do nothing for operation type " + response.getType());
                break;
        }
        // log.end();
        return output;
    }

    public void setData(Map<String, Map<String, Object>> data) {
        this.data = data;
    }

    public void setGroupName(String groupName) {
        this.groupName = groupName;
    }

    public void setMaster(int master) {
        this.master = master;
    }

    public void setMasterSet(boolean isMasterSet) {
        this.isMasterSet = isMasterSet;
    }

    /**
     * @see LifeCycleController#startFc()
     */
    public void startFc() throws IllegalLifeCycleException {
        log.call();
        configurationService = new ConfigurationService(log);
        fcState = LifeCycleController.STARTED;
        startServer();
    }

    /**
     * Start the registry server
     * 
     */
    public void startServer() {
        isStarted = true;
        // Set up the threads
        socketServerThread = new SocketServerThread(this);
        tribeThread = new TribeThread(this);
        electMasterThread = new ElectMasterThread(this);
        try {
            channel.join(new GroupIdentifier(groupName));
            // Wait the group membership to connect the other peer
            Thread.sleep(1000);
            electMasterThread.start();
            tribeThread.start();
            // Wait for the electMasterThread to set up the configuration
            electMasterThread.join();
            socketServerThread.start();
        } catch (Exception e) {
            log.error(e);
        }
    }

    /**
     * @see LifeCycleController#stopFc()
     */
    public void stopFc() throws IllegalLifeCycleException {
        log.start();
        fcState = LifeCycleController.STOPPED;
        stopServer();
        log.end();
    }

    /**
     * Stop the Registry server
     * 
     */
    public void stopServer() {
        isStarted = false;

        tribeThread.terminate();
        socketServerThread.terminate();
        try {
            AbstractReliableFifoChannel abstractReliableFifoChannel = TcpChannelPool
                .getChannelPool().getChannel(
                    channel.getLocalMembership().getAddress());
            TcpChannelPool.getChannelPool().removeChannelFromPool(
                abstractReliableFifoChannel);

            channel.quit();
            gms.stop();
            discovery.kill();

            discovery.join();
            tribeThread.join();
            socketServerThread.join();
            gms = null;
            discovery = null;
            channel = null;
        } catch (Exception e) {
            // Do nothing
        }
    }

    /**
     * @see BindingController#unbindFc(String)
     */
    public void unbindFc(String arg0) throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        if (arg0.equals(InterfaceNamesHelper.LOGGER_NAME)) {
            logger = null;
        } else {
            throw new NoSuchInterfaceException(arg0);
        }
    }

    public boolean isFullyUpdated() {
        return fullyUpdated;
    }

    public ConfigurationService getConfigurationService() {
        return configurationService;
    }

}
