/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: StateStorageTest.java 15:35:28 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.systemstate;

import java.io.File;
import java.util.List;

import junit.framework.TestCase;

/**
 * Class used to test the StateStorage class
 * 
 * @author ddesjardins - eBMWebsourcing
 * @author alouis - eBMWebsourcing
 */
public class StateStorageTest extends TestCase {

    /**
     * Base directory of the project
     */
    private String baseDir;

    private StateStorage stateStorage;

    private ComponentState c = new ComponentState("c", "iURL", "zURL", "iS",
        "lS");;

    private ServiceAssemblyState sa = new ServiceAssemblyState("sa", "iURL",
        "zURL", "lS");;

    private SharedLibraryState sl = new SharedLibraryState("sl", "iURL", "zURL");;

    /**
     * Defines the path used to store the states
     */
    protected void setUp() throws Exception {
        super.setUp();
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);

        System.setProperty("petals.home", new File(baseDir + "target")
            .getAbsolutePath());
        new File(baseDir + "target" + File.separator + "lib").mkdir();
        new File(baseDir + "target" + File.separator + "lib" + File.separator
            + "petals-1.0-SNAPSHOT.jar").mkdir();
        new File(baseDir + "target" + File.separator + "conf").mkdir();

        File stateFile = File.createTempFile("systemstate", ".xml");
        stateFile.deleteOnExit();
        File xsdFile = new File(baseDir.replace("container", "assembly")
            + "src" + File.separator + StateStorage.SCHEMA_FILE_PATH);

        stateStorage = new StateStorage();
        stateStorage.file = stateFile;
        stateStorage.schemaFile = xsdFile;
    }

    public void tearDown() {
        System.clearProperty("petals.home");
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.init()'
     */
    public void testInit() {
        // init must failed if file is null
        stateStorage.file = null;

        boolean exception = true;
        try {
            stateStorage.init();
            exception = false;
        } catch (Exception e) {
            // nothing
        }
        assertTrue(exception);
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.init()'
     */
    public void testInit2() {
        // init must failed if file is incorrect - wrong element
        stateStorage.file = new File(baseDir + File.separator + "target"
            + File.separator + "test-classes" + File.separator + "systemstate"
            + File.separator + "wrongSystemState.xml");
        assert (stateStorage.file.exists());

        boolean exception = true;
        try {
            stateStorage.init();
            exception = false;
        } catch (Exception e) {
            // nothing
        }
        assertTrue(exception);

        stateStorage.file = new File(baseDir + File.separator + "target"
            + File.separator + "test-classes" + File.separator + "systemstate"
            + File.separator + "wrongSystemState.xml");
        assert (stateStorage.file.exists());
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.init()'
     */
    public void testInit3() {
        // init must failed if file is incorrect - wrong attribute
        stateStorage.file = new File(baseDir + File.separator + "target"
            + File.separator + "test-classes" + File.separator + "systemstate"
            + File.separator + "wrongSystemState.xml");
        assert (stateStorage.file.exists());

        boolean exception = true;
        try {
            stateStorage.init();
            exception = false;
        } catch (Exception e) {
            // nothing
        }
        assertTrue(exception);
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.init()'
     */
    public void testInit4() throws Exception {
        // init must success if file is correct
        stateStorage.file = new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "systemstate" + File.separator
            + "goodSystemState.xml");
        assertTrue(stateStorage.file.exists());
        stateStorage.init();
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.createComponentStateHolder(String,
     * String, String, String, String)'
     */
    public void testCreateAndRetreiveComponent() throws Exception {
        stateStorage.init();

        ComponentState cc = stateStorage.createComponentStateHolder(
            c.getName(), c.getInstallURL(), c.getArchiveURL(), c
                .getInstallState(), c.getLifecycleState());

        // test the result of the call
        assertTrue(c.equals(cc));

        // test that the element was saved in the file
        // by reloading the systemState file.
        stateStorage.init();
        List<ComponentState> l = stateStorage.retrieveAllComponentStates();
        assertTrue(l != null);
        assertTrue(l.size() == 1);
        assertTrue(c.equals(l.get(0)));
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.createServiceAssemblyStateHolder(String,
     * String, String, String)'
     */
    public void testCreateAndRetreiveServiceAssembly() throws Exception {
        stateStorage.init();

        ServiceAssemblyState cc = stateStorage
            .createServiceAssemblyStateHolder(sa.getName(), sa.getInstallURL(),
                sa.getArchiveURL(), sa.getLifecycleState());

        // test the result of the call
        assertTrue(sa.equals(cc));

        // test that the element was saved in the file
        // by reloading the systemState file.
        stateStorage.init();
        List<ServiceAssemblyState> l = stateStorage
            .retrieveAllServiceAssemblyStates();
        assertTrue(l != null);
        assertTrue(l.size() == 1);
        assertTrue(sa.equals(l.get(0)));
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.createSharedLibraryStateHolder(String,
     * String, String)'
     */
    public void testCreateAndRetreiveSharedLibrary() throws Exception {
        stateStorage.init();

        SharedLibraryState cc = stateStorage.createSharedLibraryStateHolder(sl
            .getName(), sl.getInstallURL(), sl.getArchiveURL());

        // test the result of the call
        assertTrue(sl.equals(cc));

        // test that the element was saved in the file
        // by reloading the systemState file.
        stateStorage.init();
        List<SharedLibraryState> l = stateStorage
            .retrieveAllSharedLibraryStates();
        assertTrue(l != null);
        assertTrue(l.size() == 1);
        assertTrue(sl.equals(l.get(0)));
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.updateComponentInstallationState(String,
     * String)'
     */
    public void testUpdateComponentInstallationState() throws Exception {
        stateStorage.init();

        stateStorage.createComponentStateHolder(c.getName(), c.getInstallURL(),
            c.getArchiveURL(), c.getInstallState(), c.getLifecycleState());

        stateStorage.updateComponentInstallationState(c.getName(),
            "uninstalled");

        // test that the state is modified
        // by reloading the systemState file.
        stateStorage.init();
        List<ComponentState> l = stateStorage.retrieveAllComponentStates();
        assertTrue(l != null);
        assertTrue(l.size() == 1);
        assertTrue("uninstalled".equals(l.get(0).getInstallState()));
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.updateComponentLifeCycleState(String,
     * String)'
     */
    public void testUpdateComponentLifeCycleState() throws Exception {
        stateStorage.init();

        stateStorage.createComponentStateHolder(c.getName(), c.getInstallURL(),
            c.getArchiveURL(), c.getInstallState(), c.getLifecycleState());

        stateStorage.updateComponentLifeCycleState(c.getName(), "stopped");

        // test that the state is modified
        // by reloading the systemState file.
        stateStorage.init();
        List<ComponentState> l = stateStorage.retrieveAllComponentStates();
        assertTrue(l != null);
        assertTrue(l.size() == 1);
        assertTrue("stopped".equals(l.get(0).getLifecycleState()));
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.updateServiceAssemblyState(String,
     * String)'
     */
    public void testUpdateServiceAssemblyState() throws Exception {
        stateStorage.init();

        stateStorage.createServiceAssemblyStateHolder(c.getName(), c
            .getInstallURL(), c.getArchiveURL(), c.getLifecycleState());

        stateStorage.updateServiceAssemblyState(c.getName(), "stopped");

        // test that the state is modified
        // by reloading the systemState file.
        stateStorage.init();
        List<ServiceAssemblyState> l = stateStorage
            .retrieveAllServiceAssemblyStates();
        assertTrue(l != null);
        assertTrue(l.size() == 1);
        assertTrue("stopped".equals(l.get(0).getLifecycleState()));
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.deleteComponentStateHolder(String)'
     */
    public void testDeleteComponentStateHolder() throws Exception {
        stateStorage.init();

        stateStorage.createComponentStateHolder(c.getName(), c.getInstallURL(),
            c.getArchiveURL(), c.getInstallState(), c.getLifecycleState());

        stateStorage.deleteComponentStateHolder(c.getName());

        // test that the state is deleted
        // by reloading the systemState file.
        stateStorage.init();
        List<ComponentState> l = stateStorage.retrieveAllComponentStates();
        assertTrue(l != null);
        assertTrue(l.size() == 0);
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.deleteServiceAssemblyStateHolder(String)'
     */
    public void testDeleteServiceAssemblyStateHolder() throws Exception {
        stateStorage.init();

        stateStorage.createServiceAssemblyStateHolder(c.getName(), c
            .getInstallURL(), c.getArchiveURL(), c.getLifecycleState());

        stateStorage.deleteServiceAssemblyStateHolder(c.getName());

        // test that the state is deleted
        // by reloading the systemState file.
        stateStorage.init();
        List<ServiceAssemblyState> l = stateStorage
            .retrieveAllServiceAssemblyStates();
        assertTrue(l != null);
        assertTrue(l.size() == 0);
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.systemstate.StateStorage.deleteSharedLibraryStateHolder(String)'
     */
    public void testDeleteSharedLibraryStateHolder() throws Exception {
        stateStorage.init();

        stateStorage.createSharedLibraryStateHolder(c.getName(), c
            .getInstallURL(), c.getArchiveURL());

        stateStorage.deleteSharedLibraryStateHolder(c.getName());

        // test that the state is deleted
        // by reloading the systemState file.
        stateStorage.init();
        List<SharedLibraryState> l = stateStorage
            .retrieveAllSharedLibraryStates();
        assertTrue(l != null);
        assertTrue(l.size() == 0);
    }

}
